pro ssw_chianti_service, query, qfile=qfile, $
   foreground=foreground , background=background, debug=debug,$
   no_param=no_param
;+
;   Name: ssw_chianti_service
;
;   Purpose: manage SSW/WWW services for Chianti package
;
;   Input Paramters:
;      query - POST or GET query stream
;
;   Keyword Parameters:
;      qfile - POST or GET query file (alternative to query input)
;      foreground - if set, run in foreground (override QUERY setting)
;      background - if set, background (add to query pending 
;
;   History:
;      12-June-2003 - S.L.Freeland - from ssw_service.pro
;      (need template...)
;-

debug=keyword_set(debug)
toppath=get_logenv('path_http') ; local path for http<ssw> comm
tophttp=get_logenv('top_http')  ; url equivilent
if toppath eq '' then begin     ; lmsal sswserver defaults
   set_logenv,'path_http','/net/diapason/www1/htdocs/solarsoft/ssw_client'
   set_logenv,'top_http','http://www.lmsal.com/solarsoft/ssw_client'
endif  

;  ---- get the query, by query string or query-filename ---
case 1 of 
   data_chk(qfile,/string): begin
      box_message,['QFILE: '+qfile,rd_tfile(qfile)] 
      info=url_decode(qfile=qfile)
      break_file,qfile,ll,pp,ff,ee,vv
      
      outdir=concat_dir('$path_http','data')
      help,outdir,http_names(outdir)
      response_file=qfile+'_response.html'        ; for cosec_mode=2 output
   endcase

   data_chk(query,/string): info=query
   else: begin 
      box_message,'Require QUERY string or QUERY file
      return
   endcase
endcase

background=gt_tagval(info,/ssw_chianti_service_execution,missing='') eq 'background' $
   or keyword_set(background)

foreground=keyword_set(foreground) or (1-background)

help,info,/str
service=gt_tagval(info,/SSW_CHIANTI_SERVICE,missing='')

if total(strpos(service,' ')) ge 0 then $
   service=strlowcase(ssw_strsplit(service,' ',/head))
netdisp=gt_tagval(info,/NETSCAPE_DISPLAY, missing='')

s1=gt_tagval(info,/param1_sample,missing='')
s2=gt_tagval(info,/param2_sample,missing='')

p1=([gt_tagval(info,/param1,missing=''),s1])(s1 ne '')

p2=([gt_tagval(info,/param2,missing=''),s2])(s2 ne '')

; extra is passed verbatim to service
extra=strcompress(strlowcase(gt_tagval(info,/EXTRA,missing='')),/remove)
cosec_mode=get_logenv('cosec_mode')
cosec_mode=gt_tagval(info,/cosec_mode,missing=cosec_mode)
help,cosec_mode

textra=extra+','

if strpos(textra,'xsize=') ne -1 then $
   xsize=strextract(textra,'xsize=',',') else xsize='600'

if strpos(textra,'ysize=') ne -1 then $
   ysize=strextract(textra,'ysize=',',') else ysize=xsize

break_url,p1,server1,path1,files1,ftp=ftp,http=http
break_url,p2,server2,path2,files2,ftp=ftp,http=http

post2=gt_tagval(info,/URL_DESTINATION, missing='')

;   non-default input parameters
; Chianti specific possibilities 
; (seems like I could generalize/loop {ioneq/dem/abund} select w/execute since code
;  is ~identical but not right this second...
 
chidbase=concat_dir('$SSW_CHIANTI','dbase')   ; map to local path

; ION Equalization file defintions
defion='mazzotta_etal'                        ; default selection
allioneq=loc_file(path=concat_dir(chidbase,'ioneq'),'*ioneq')
useioneq=strlowcase(gt_tagval(info,/IONEQ,missing=defion))
ssion=where(strpos(allioneq,useioneq(0)) ne -1,icnt)
if icnt eq 0 then ssion=where(strpos(allioneq,defion) ne -1)
ioneq=allioneq(ssion(0)>0)
 
; DEM
defion='flare'                               ; default selection
alldem=loc_file(path=concat_dir(chidbase,'dem'),'*dem')
usedem=strlowcase(gt_tagval(info,/DEM,missing=defion))
ssdem=where(strpos(alldem,usedem(0)) ne -1, dcnt)
if dcnt eq 0 then ssdem=where(strpos(alldem,defdem) ne -1)
dem=alldem(ssdem(0)>0)

; ABUNDANCE
defabund='sun_coronal'              ; default
allabund=loc_file(path=concat_dir(chidbase,'abundance'),'*abund')
useabund=strlowcase(gt_tagval(info,/ABUND,missing=defion))
ssabund=where(strpos(allabund,useabund(0)) ne -1, acnt)
if acnt eq 0 then ssabund=where(strpos(allabund,defabund) ne -1)
abund=allabund(ssabund(0)>0)

box_message,[ioneq,dem,abund]          ; loud for now

; Most things via _EXTRA - but here are some specifics
; rationalize 'sngl_ion' which may be entered in strange ways (until client-side checks..)
sngl_ion=gt_tagval(info,/SNGL_ION,missing='')   ; check for one or more
sngl_ion=str_replace(str_replace(str_replace(sngl_ion,'[',''),']',''),'"','')
sngl_ion=str2arr(str_replace(sngl_ion,"'",""))
sngl_ion=str_replace(str_replace(sngl_ion,' ','_'),'-','_')

mini=gt_tagval(info,/MINI,missing='1.e13')
wrange=gt_tagval(info,/WRANGE,missing='10.,20')
wrange=str_replace(str_replace(wrange,'[',''),']','')

help,service(0),background,p1,p2, xsize, ysize

no_param=keyword_set(no_param)   ; default prepends expected variable names

ps1=(['param1=',''])(no_param)
ps2=(['param2=',''])(no_param)
ext='extra='
if debug then stop,'before background/forground case'

if background then begin
   box_message,'background task'
   outfile=concat_dir('$path_http','www_pending')
   file_append,outfile,rd_tfile(qfile),/new
endif else begin                              ; else running in forground
  
;  check for structure/genx input
   copyok=0
   if strpos(p1,'geny') ne -1 then begin 
      locname=http_names(p1)
      if file_exist(locname) then box_message,'File is already local' else begin
         sock_copy,p1,out_dir=outdir
      endelse
      estat=execute('restgenx,file=locname,synstruct')
box_message,locname
      copyok=file_exist(locname)
   endif 

   box_message,'foreground'
   case service(0) of
      'synthetic': begin 
         if is_number(p1) and is_number(p2) then begin 
            estring='ch_synthetic,min([p1,p2]),max([p1,p2]),'+ $
                    'ioneq=ioneq,dem=dem,sngl_ion=sngl_ion,output=synstruct' + $
                    (["",',' + extra])(extra ne '')
print,estring,sngl_ion
help,sngl_ion
            estat=execute(estring)
            if estat and data_chk(synstruct,/struct) then begin 
               ofile=concat_dir(outdir,ff+'.geny')
               savegenx, synstruct,file=ofile
               qfile=ps1+url_encode(http_names(ofile))     ; return genx url
            endif else qfile=ps1+'ch_synthetic - execution problem' 
         endif else qfile=ps1+'ch_synthetic - wavelength problem'
      endcase

      'line_list': begin
         if copyok then begin 
            if required_tags(synstruct,'lines,date,version,dem') then begin 
               ascfile=concat_dir(outdir,ff+'ch_ll.ascii')
               estring='ch_line_list,synstruct,ascfile,/ascii,'+$
                  'mini=mini,abundfile=abund' + $ 
                 (["",',' + extra])(extra ne '')
               estat=execute(estring)
               if estat and file_exist(ascfile) then begin
                   ofile=concat_dir(outdir,ff+'line_list.html')
                   estat=execute('file_append,ofile,ssw_chianti_ll2html(ascfile),/new')
                   qfile=ps1+url_encode(http_names(ofile))     ; return html url
               endif else qfile=ps1+'ch_line_list - problem with ascii listing'
            endif else qfile=ps1+'ch_line_list - problem with genx read/structure'
         endif else qfile=ps1+'ch_line_list - problem with input genx transfer'
      endcase
      'make_spec': begin 
          if copyok and data_chk(synstruct,/struct) then begin 
             estring='make_chianti_spec,synstruct,lambda,specstruct,' + $
             'wrange=['+wrange+ '],abund=abund'+ $
              (["",',' + extra])(extra ne '')
             estat=execute(estring)
             if estat and data_chk(specstruct,/struct)  then begin 
               ofile=concat_dir(outdir,ff+'.geny')
               savegenx, specstruct,file=ofile
               qfile=ps1+url_encode(http_names(ofile))     ; return genx url
             endif else qfile=ps1+'make_chianti_spec - execution problem' 
          endif else qfile=ps1+'make_chianti_spec - problem with input file'
      endcase
      'plot_continuum': begin
      endcase
      'plot_synthetic': begin
          wdef,xx,768,512,/zbuffer
          linecolors
          estring="plot_goes,back=11,color=7,gcolor=150,'"+ p1+"','" +p2+"'" + $
             tstyle + trange
          estring=estring+ (["",',' + extra])(extra ne '')
print,'>>> ' + estring
             estat=execute(estring)
             ofile=concat_dir(outdir,ff+'.png')
             zbuff2file2,ofile
             qfile=ps1+url_encode(http_names(ofile))

      endcase

      else: begin
      endcase
   endcase
   if post2 ne '' and cosec_mode ne '2' then begin
      box_message,'Posting result...'
      if debug then stop,'post'
      box_message,['POST2> ' + post2,'QUERY> ' + qfile]
      ssw_post_query,post2,qfile, status=status      ,/get
      if netdisp ne '' then begin 
         estring='netscape_control,"'+qfile+'",/openurl,display="'+netdisp+'"'
         estat=execute(estring)
      endif
   endif
      case 1 of
      cosec_mode eq '2' or cosec_mode eq '3': begin
         qdecode=url_decode(qfile)
         outvals=strarr(n_tags(qdecode))
         params=tag_names(qdecode)
         for tt=0,n_tags(qdecode)-1 do $
               outvals(tt)=strtrim(qdecode.(tt),2)
         if data_chk(qdecode,/struct) then begin
            xmlout=['<result>',$
                    '<param name="'+ params +'">' + $
                     outvals + '</param>',           $
                    '</result>' ]

            help,qdecode,/str
            print,http_names(response_file)
            if debug then stop
            file_append,response_file,['<html><body>',$
                 xmlout,               '</body></html>']
          endif else box_message,'Problem with output...'
      endcase
      else:
   endcase
endelse

return
end
