function ssw_cosec_service, service_name, param1, param2,extra=extra, $
   debug=debug, display=display
;+
;
;   Name: ssw_cosec_service  
;
;   Purpose: ssw client interface to remote CoSEC SSW services
;
;   Input Parameters 
;      service_name - name of desired service
;      param1 - input parameter -> service (time, url.. as appropriate)
;      param2 - sometimes optional second parameter    
;
;   Keyword Parameters:
;      extra - optional parameter keyword string accepted by underlying service 
;      display - optional - if SET and service output is a graphics-url then 
;                transfer and display in local ssw client
;
;   Calling Examples:
;      url=ssw_cosec_service('plot_ace','1-nov-2003','5-nov-2003',extra='/particles')
;      url=ssw_cosec_service('plot_goes','1-nov-2003','5-nov-2003')
;      url=ssw_cosec_service('pfss','1-nov-2003',extra='nlines=100,mag=3,/earth_view') 
;      urls=ssw_cosec_service('bracket_event',<time>,extra='mdi') ;      
;      urls=ssw_cosec_service('bracket_event',<time>,extra='eit') ;
;      (output from previous two may be piped->sock_copy for example)
;
;   Method:
;      set up and call requested service
;              
;      
;   Restrictions:
;      proto-version - no X-reference to CoSEC data typing ontology 
;                      so Caveat Emptor on service:param{1,2 } type matching
;                      Only a subset of services offered (since if you can run this,
;                      you can likely run the real thing on your client....) 
;      Future "released" version needs to access CoSEC ontology dbase
;              directly
;
;      Services (via service_name) restricted to:
;         plot_goes aka plot_goes xrays - GOES xrays for user input time range  
;         plot_ace - plot some ACE data (try extra='/WIND' or extra='/PARTICLES' )
;         bracket_event - return urls before and after some event  
;                         (try extra='eit' or extra='mdi')
;
;   Motivation:
;      offer a number of restricted services to clients who may not have a 
;      dbase local (but you could...) and to demo ssw/service interface  
;      possibly useful for IDL VM applications...)
;
if n_params() lt 2 then begin 
   box_message,'Need at least a service ID/name and one parameter..'
   return,''
endif

debug=keyword_set(debug)

if n_elements(param2) eq 0 then param2=''

service=strlowcase(service_name(0))
handler='ssw_service_'

if n_elements(extra) eq 0 then extra=''
extra=(['','&extra='+url_encode(extra(0))])(extra(0) ne '')
case 1 of
   is_member(service,'pfss,draw_field,field'): begin
      ssw_service='draw_field'
      handler='ssw_pfss_service_'
   endcase
   is_member(service,'goes,plot_goes,plot_goes_xrays'): ssw_service='plot_goes_xrays'
   is_member(service,'particles,protons,plot_goes_protons'): ssw_service='plot_goes_protons'
   is_member(service,'ace,plot_ace'): begin
      if not data_chk(extra,/string) then extra='/wind'
       ssw_service='plot_ace'
   endcase
   is_member(service,'bracket_event'): begin
      ssw_service='best_flare_fd'
      if param2 eq '' then param2=param1
      if extra ne '' then $
         extra=str_replace(str_replace(extra,'extra=','inst_wave='),'mdi','mag')
   endcase
   else: begin 
      box_message,'Specified service >> ' + service + ' Not recognized...'
      return,''
   endcase
endcase

cosec_top='http://www.lmsal.com/solarsoft/cgi-diapason/'
servurl=cosec_top+handler+ssw_service+'.sh?param1='+url_encode(param1(0)) + $
        (['&param2='+url_encode(param2),''])(param2 eq '') + extra + $
        '&cosec_mode=2
        
if debug then stop,servurl

sock_list, servurl,servoutput,/cgi  ; execute CGI service 

ssout=where(strpos(servoutput,'PARAM') ne -1,sscnt)

if sscnt gt 0 then begin 
   params=servoutput(ssout)
stop

endif

retval=servoutput
return,retval
end 





