pro ssw_pfss_service, query, qfile=qfile, $
   foreground=foreground , background=background, debug=debug,$
   no_param=no_param
;+
;   Name: ssw_pfss_service
;
;   Purpose: manage SSW/WWW services for PFSS suite
;
;   Input Paramters:
;      query - POST or GET query stream
;
;   Keyword Parameters:
;      qfile - POST or GET query file (alternative to query input)
;      foreground - if set, run in foreground (override QUERY setting)
;      background - if set, background (add to query pending 
;
;   History:
;      25-October-2001 - S.L.Freeland
;       7-November-2001 - S.L.Freeland - enable plot_map/plot_goes
;                         optional client display
;      24-May-2002 - S.L.Freeland - add "no_param" keyword and made
;                    prepending "param1=<url>" the default
;       4-feb-2004 - S.L.Freeland - from ssw_service.pro, make PFSS service wrapper
;-

debug=keyword_set(debug)
set_logenv,'path_http','/net/diapason/www1/htdocs/solarsoft/ssw_client'
set_logenv,'top_http','http://www.lmsal.com/solarsoft/ssw_client'

box_message,'test',nbox=5

case 1 of 
   data_chk(qfile,/string): begin
      box_message,['QFILE: '+qfile,rd_tfile(qfile)] 
      info=url_decode(qfile=qfile)
      break_file,qfile,ll,pp,ff,ee,vv
      
      outdir=concat_dir('$path_http','data')
      help,outdir,http_names(outdir)
      response_file=qfile+'_response.html'        ; for cosec_mode=2 output
   endcase

   data_chk(query,/string): info=query
   else: begin 
      box_message,'Require QUERY string or QUERY file
      return
   endcase
endcase

background=gt_tagval(info,/ssw_service_execution,missing='') eq 'background' $
   or keyword_set(background)

foreground=keyword_set(foreground) or (1-background)

help,info,/str
service=gt_tagval(info,/SSW_PFSS_SERVICE,missing='')

if total(strpos(service,' ')) ge 0 then $
   service=strlowcase(ssw_strsplit(service,' ',/head))
netdisp=gt_tagval(info,/NETSCAPE_DISPLAY, missing='')

s1=gt_tagval(info,/param1_sample,missing='')
s2=gt_tagval(info,/param2_sample,missing='')

p1=([gt_tagval(info,/param1,missing=''),s1])(s1 ne '')

p2=([gt_tagval(info,/param2,missing=''),s2])(s2 ne '')

; extra is passed verbatim to service
extra=strcompress(strlowcase(gt_tagval(info,/EXTRA,missing='')),/remove)
cosec_mode=get_logenv('cosec_mode')
cosec_mode=gt_tagval(info,/cosec_mode,missing=cosec_mode)
help,cosec_mode

start_radius=gt_tagval(info,/start_radius,missing='[1,2.5]')

; for plot services, assume exact time access for comparisons
; the following 2 lines set utplot timerange and XSTYLE defaults
; (XSTYLE or TIMERANGE set via EXTRA override this default)
tstyle=(['',',/XSTYLE'])(strpos(strupcase(extra),'XSTYLE=') eq -1) 
trange=(['',',TIMERANGE=[p1,p2]'])(strpos(strupcase(extra),'TIMERANGE=') eq -1)

exp_times_in=float(str2arr(gt_tagval(info,/exp_times_in,missing=[0,0])))
exp_times_out=float(str2arr(gt_tagval(info,/exp_times_out,missing=[0,0])))

textra=extra+','

if strpos(textra,'xsize=') ne -1 then $
   xsize=strextract(textra,'xsize=',',') else xsize='600'

if strpos(textra,'ysize=') ne -1 then $
   ysize=strextract(textra,'ysize=',',') else ysize=xsize

break_url,p1,server1,path1,files1,ftp=ftp,http=http
break_url,p2,server2,path2,files2,ftp=ftp,http=http

post2=gt_tagval(info,/URL_DESTINATION, missing='')

;   non-default input parameters
inst_wave=gt_tagval(info,/INST_WAVE,missing='')
image_reference=gt_tagval(info,/IMAGE_REFERENCE,missing='')
fov=str2number(gt_tagval(info,/FOV)) > 60  
nlines=str2number(gt_tagval(info,/NLINES)) < 300     ; limit number of calculations
uniq_id=gt_tagval(info,/UNIQ_ID,missing='')

if uniq_id eq '' then uniq_id=p2 

help,service(0),background,p1,p2, xsize, ysize, inst_wave

no_param=keyword_set(no_param)   ; default prepends expected variable names

ps1=(['param1=',''])(no_param)
ps2=(['param2=',''])(no_param)
ext='extra='
if debug then stop,'before background/forground case'

if background then begin
   box_message,'background task'
   outfile=concat_dir('$path_http','www_pending')
   file_append,outfile,rd_tfile(qfile),/new
endif else begin 

   box_message,'foreground'
   case service(0) of
      'draw_field': begin
          if strpos(p1,'http:') eq -1 then begin 
             index=p1 
          endif else begin  
             loc=http_names(p1)
             if file_exist(loc) then begin 
                box_message,'File is Local...'
                estat=execute('mreadfits,loc,index,data')
box_message,loc
help,estat
             endif else  begin 
                estat=execute('ssw_url2data,p1,index,data')
             endelse
          endelse
help,index,data
          
;          if data_chk(index,/struct) then begin 
             if keyword_set(image_reference) then begin 
                local=http_names(image_reference)
                if file_exist(local) then begin
                   box_message,'Image reference local, no transfer required'
                   restgenx,file=local,image_reference,refdata
                endif
             endif
             bbox=""             ; line rooting  bounding box (init to null) 
             refstr=""           ; optional reference image (init to null) 
             if required_tags(image_reference,'naxis1,naxis2,crpix1') then begin  
                refind=image_reference
                if n_elements(refdata) eq 0 then refdata=bytarr(refind.naxis1,refind.naxis2)
                ;index2map,refind,refdata,refmap
                ;sub_map,refmap,smap,ref_map=refmap
                ;map=temporary(smap)
                ;bbox="bbox=map"
                refstr=",image_reference=refind"
             endif
             set_plot,'z'
             snlines='nlines='+strtrim(nlines,2)
             srad=',start_radius='+start_radius
             estring='ssw_index2pfss_multi,index,/earth_view,pfssindex,pfssdata,' + $
                    snlines + srad + bbox + refstr + (["",',' + extra])(extra ne '') 
             box_message,estring
             estat=execute(estring)
print,estring
help,estat,pfssdata
             ofile=concat_dir(outdir,ff+'.png')
             tvlct,rr,gg,bb,/get
             tvlct,rx,gx,bx,/get
             dtemp=!d.name
             if data_chk(pfssdata,/nimage) ge 1 then begin 
                wdef,xx,image=pfssdata,/zbuffer
                pfssdlc=ssw_pfss2linecolors(pfssdata) 
help,'pfss2linecolors'
                loadct,0
                linecolors
                tv,pfssdlc
                tvlct,rx,gx,bx,/get
             endif
             zbuff2file2,ofile
             tvlct,rr,gg,bb    
             qfile=ps1+url_encode(http_names(ofile))
print,qfile
;          endif
      endcase
      else: begin
      endcase
   endcase
      if netdisp ne '' and file_exist(ofile) then begin 
         estring='netscape_control,"'+$
            http_names(ofile)+'",/openurl,display="'+netdisp+'"'
         estat=execute(estring)
      endif
   if post2 ne '' and cosec_mode ne '2' then begin
      box_message,'Posting result...'
      if debug then stop,'post'
      box_message,['POST2> ' + post2,'QUERY> ' + qfile]
      ssw_post_query,post2,qfile, status=status      ,/get
   endif
   case 1 of
      cosec_mode eq '2' or cosec_mode eq '3': begin
         qdecode=url_decode(qfile)
help,qfile,qdecode
         outvals=strarr(n_tags(qdecode))
         params=tag_names(qdecode)
         for tt=0,n_tags(qdecode)-1 do $
               outvals(tt)=strtrim(qdecode.(tt),2)
         if data_chk(qdecode,/struct) then begin
            xmlout=['<result>',$
                    '<param name="'+ params +'">' + $
                     outvals + '</param>',           $
                    '</result>' ]
box_message,xmlout
            help,qdecode,/str
            print,http_names(response_file)
            if debug then stop
            file_append,response_file,['<html><body>',$
                 xmlout,               '</body></html>']
          endif else box_message,'Problem with output...'
      endcase
      else:
   endcase
endelse

return
end
