pro ssw_service, query, qfile=qfile, $
   foreground=foreground , background=background, debug=debug,$
   no_param=no_param
;+
;   Name: ssw_service
;
;   Purpose: manage SSW/WWW services
;
;   Input Paramters:
;      query - POST or GET query stream
;
;   Keyword Parameters:
;      qfile - POST or GET query file (alternative to query input)
;      foreground - if set, run in foreground (override QUERY setting)
;      background - if set, background (add to query pending 
;
;   History:
;      25-October-2001 - S.L.Freeland
;       7-November-2001 - S.L.Freeland - enable plot_map/plot_goes
;                         optional client display
;      24-May-2002 - S.L.Freeland - add "no_param" keyword and made
;                    prepending "param1=<url>" the default
;      28-jul-2004 - some param protection, add rebin_graphic service
;       1-sep-2004 - S.L.Freeland - trace movie service
;       1-sep-2005 - S.L.Freeland - plot_rstn_srs (radio/spectrum)
;       7-sep-2005 - S.L.Freeland - get_cmelist (lasco @ gsfc -> VOTable)
;      19-oct-2005 - S.L.Freeland - NO_XML option/url_type
;-

debug=keyword_set(debug)
set_logenv,'path_http','/net/diapason/www1/htdocs/solarsoft/ssw_client'
set_logenv,'top_http','http://www.lmsal.com/solarsoft/ssw_client'

case 1 of 
   data_chk(qfile,/string): begin
      box_message,['QFILE: '+qfile,rd_tfile(qfile)] 
      info=url_decode(qfile=qfile)
      break_file,qfile,ll,pp,ff,ee,vv
      
      outdir=concat_dir('$path_http','data')
      help,outdir,http_names(outdir)
      response_file=qfile+'_response.html'        ; for cosec_mode=2 output
   endcase

   data_chk(query,/string): info=query
   else: begin 
      box_message,'Require QUERY string or QUERY file
      return
   endcase
endcase

background=gt_tagval(info,/ssw_service_execution,missing='') eq 'background' $
   or keyword_set(background)

foreground=keyword_set(foreground) or (1-background)

help,info,/str
service=gt_tagval(info,/SSW_SERVICE,missing='')

if total(strpos(service,' ')) ge 0 then $
   service=strlowcase(ssw_strsplit(service,' ',/head))
netdisp=gt_tagval(info,/NETSCAPE_DISPLAY, missing='')

s1=gt_tagval(info,/param1_sample,missing='')
s2=gt_tagval(info,/param2_sample,missing='')

p1=(([gt_tagval(info,/param1,missing=''),s1])(s1 ne ''))(0)

p2=(([gt_tagval(info,/param2,missing=''),s2])(s2 ne ''))(0)

; extra is passed verbatim to service
extra=strcompress(strlowcase(gt_tagval(info,/EXTRA,missing='')),/remove)
cosec_mode=get_logenv('cosec_mode')
cosec_mode=gt_tagval(info,/cosec_mode,missing=cosec_mode)
help,cosec_mode

; for plot services, assume exact time access for comparisons
; the following 2 lines set utplot timerange and XSTYLE defaults
; (XSTYLE or TIMERANGE set via EXTRA override this default)
tstyle=(['',',/XSTYLE'])(strpos(strupcase(extra),'XSTYLE=') eq -1) 
trange=(['',',TIMERANGE=[p1,p2]'])(strpos(strupcase(extra),'TIMERANGE=') eq -1)

exp_times_in=float(str2arr(gt_tagval(info,/exp_times_in,missing=[0,0])))
exp_times_out=float(str2arr(gt_tagval(info,/exp_times_out,missing=[0,0])))

textra=extra+','

if strpos(textra,'xsize=') ne -1 then $
   xsize=strextract(textra,'xsize=',',') else xsize='600'

if strpos(textra,'ysize=') ne -1 then $
   ysize=strextract(textra,'ysize=',',') else ysize=xsize

break_url,p1,server1,path1,files1,ftp=ftp,http=http
break_url,p2,server2,path2,files2,ftp=ftp,http=http

post2=gt_tagval(info,/URL_DESTINATION, missing='')

;   non-default input parameters
inst_wave=(gt_tagval(info,/INST_WAVE,missing=''))(0)
image_reference=(gt_tagval(info,/REFERENCE,missing=''))(0)
fov=(str2number(gt_tagval(info,/FOV)) > 60  )(0)
time_unit=(gt_tagval(info,/TIME_UNIT,missing='minutes'))(0)
time_window=gt_tagval(info,/TIME_WINDOW,missing='-0,0')
uniq_id=gt_tagval(info,/UNIQ_ID,missing='')
replace=gt_tagval(info,/REPLACE,missing='') ne "No"
url_type=strlowcase(gt_tagval(info,/URL_TYPE,missing=''))
no_xml=gt_tagval(info,/NO_XML,missing='') ne ''

; general image/movie 
maxframe=gt_tagval(info,/MAXFRAME,missing=10)
minframe=gt_tagval(info,/MINFRAME,missing=2)
outsize=gt_tagval(info,/OUTSIZE,missing=512)
reverse_ct=gt_tagval(info,/REVERSE_CT,missing=0)

if uniq_id eq '' then uniq_id=p2 

help,service(0),background,p1,p2, xsize, ysize, inst_wave

no_param=keyword_set(no_param)   ; default prepends expected variable names

ps1=(['param1=',''])(no_param)
ps2=(['param2=',''])(no_param)
ext='extra='
if debug then stop,'before background/forground case'

if total(abs(exp_times_in)) gt 0 then begin
   time_window,[p1,p2],np1,np2, minutes=exp_times_in
   box_message,'Expanding Input Time Window'
   help,p1,p2,np1,np2
   p1=np1
   p2=np2
endif

threshold=gt_tagval(info,/threshold,missing='')
if threshold ne '' then threshold='threshold="'+threshold +'"'

if background then begin
   box_message,'background task'
   outfile=concat_dir('$path_http','www_pending')
   file_append,outfile,rd_tfile(qfile),/new
endif else begin 

   box_message,'foreground'
   case 1  of
      service(0) eq 'expand_times': begin 
            times=p1     
            if p2(0) ne '' then times=[p1,p2]
            time_window=$
               strtrim(str_replace(str_replace(time_window,']'),'['),2) 
            rstring=time_unit+'=['+time_window+']'
            estring='time_window,times,ot1,ot2,'+rstring $
              + (["",',' + extra])(extra ne '') 
            ot1=p1 
            ot2=p2
            estat=execute(estring)
box_message,[times,estring]
            qfile=ps1+url_encode(ot1(0))+'&'+ps2+url_encode(ot2(0))
      endcase

      service(0) eq 'build_url_list': begin 
         help,uniq_id
         if uniq_id eq '' then uniq_id='zzz'
         ofile=concat_dir(outdir,uniq_id)+'.url_list'
         file_append,ofile(0),p1,new=replace
         qfile=ps1+url_encode(http_names(ofile))
      endcase
      service(0) eq 'level0_access': begin 
         if n_elements(p1) eq 1 then p1=str2arr(p1,' ')
         case inst_wave of
            'trace': $
               qfile=ps1+'NODATA'
            else: begin 
               urls='http://'+ eit_file2url(p1)
               qfile=ps1+url_encode(arr2str(urls,' '))  
            endcase
         endcase 
      endcase
      service(0) eq 'level0_to_level1': begin 
box_message,'level0tolevel1'
         if n_elements(p1) eq 1 then p1=str2arr(p1,' ')
         break_url,p1,server1,path1,files1
         for i=0,n_elements(p1)-1 do begin
            box_message,'Transferring: ' + p1(i)
            sock_copy,p1(i),out_dir=outdir,err=err    
         endfor
         ofiles=concat_dir(outdir,files1)
print,ofiles
         if total(file_exist(ofiles)) lt n_elements(p1) then begin 
            qfile=ps1+'PROBLEM'
         endif else begin 
            case 1 of
               strpos(p1(0),'soi.') ne -1: begin 
box_message,'MDI level0->1'
                  rd_mdi,ofiles,oindex,odata
               endcase
               else: begin 
box_message,'EIT level0->1'
                  read_eit,ofiles,index,data
                  eit_prep,index,data=data,oindex,odata
               endcase
            endcase
            onames=concat_dir(outdir,'sswl1_'+files1)
help,oindex,odata
            mwritefits,oindex,odata,outfile=onames
print,onames
print,http_names(onames)
            qfile=ps1+arr2str(http_names(onames),' ') 
         endelse
      endcase
      service(0) eq 'flare_locator': begin 
box_message,'FLARE Locator'
         qfile=ps1+'dummy'+ '&'+ ps2+'dummy'
         ofiles=concat_dir(outdir,[files1,files2])
         if total(file_exist(ofiles)) eq 2 then begin 
            box_message,'Files already cached - No transfer required'
         endif else begin  
            sock_copy,p1,out_dir=outdir
            sock_copy,p2,out_dir=outdir
         endelse
         if total(file_exist(ofiles)) lt 2 then begin 
print,ofiles
            qfile=ps1+'PROBLEM'
         endif else begin 
            mreadfits,ofiles,index,data
            helio=ssw_flare_locator(index,data,oindex,odata,/despikes,/helio)
help,helio
            helio=ssw_helio2string(helio)
help,helio
            qfile=ps1+url_encode(helio)+'&'+ $
               ps2+url_encode(anytim(oindex,/vms,/trunc))
         endelse
      endcase
      service(0) eq 'latest_events_loc': begin
         set_plot,'z'
         estring='lesarc=les_archive_info(p1,p2)'
         estat=execute(estring)
         helio="Problem"
         oindex=p1
         if data_chk(lesarc,/struct) then begin 
            rec=last_nelem(lesarc(sort(lesarc.class)))
            helio=gt_tagval(rec,/HELIO)
            oindex=gt_tagval(rec,/DATE_OBS)
            help,rec,/str
         endif
         qfile=ps1+url_encode(helio)+'&'+ $
            ps2+url_encode(anytim(oindex,/vms,/trunc))
      endcase
      service(0) eq 'latest_events_url': begin 
         event=strpos(url_type,'eve') ne -1
         movie=strpos(url_type,'mov') ne -1
         sxi=strpos(url_type,'sxi') ne -1
         eit=strpos(url_type,'eit') ne -1 
         locator=strpos(url_type,'loc') ne -1 
        
         summary=strpos(url_type,'summ') ne -1 or $
           total([event,movie,sxi,eit,locator]) eq 0
         estring='lesarc=les_archive_info(p1,' + $
           'summary_url=summary,'                + $
           'event_url=event, movie_url=movie,' + $
           'sxi=sxi, eit=eit, locator_url=locator)'
box_message,estring
         estat=execute(estring)
         qfile=ps1 + url_encode(lesarc(0))
      endcase
      service(0) eq 'best_flare_fd': begin 
         case inst_wave of
            'trace':
            else: begin
               inst_wave=([inst_wave,'mdi'])(strlowcase(inst_wave) eq 'mag') 
               if inst_wave eq '' then inst_wave='eit' else $
                  inst_wave = strmid(strtrim(inst_wave,2),0,3)
               estring= inst_wave + $
                  '_closest_fd,p1,p2,fbefore,fafter,status=status'
box_message,estring
help,p1,p2
               estat=execute(estring)
               if status and estat then begin 
                  qfile=ps1+url_encode(fbefore) + '&' + $
                        ps2+url_encode(fafter)
               endif else qfile=url_encode('NODATA')
            endcase
          endcase
      endcase
      service(0) eq 'trace_data': begin 
         estring='trace_time2level1,p1,p2,outdir=outdir,'    +$
            'outfiles=outfiles,maxfiles=maxfiles'
         qfile=concat_dir(outdir,'ssw_td'+'ff.html')
         html_doc,qfile,/header
         file_append,qfile,str2html(http_names(outfiles))+'<br>'
         html_doc,qfile,/trailer
         qfile=ps1+url_encode(http_names(qfile)) 
      endcase 
      service(0) eq 'trace_movie': begin 
          phttp=get_logenv('path_http')
          thttp=get_logenv('top_http')
          set_logenv,'path_http','/DataCenter/www/htdocs/TRACE'
          set_logenv,'top_http','http://xserve.lmsal.com/TRACE'
          delvarx,topurl
          estat=execute('adjust_times,p1,p2,maxdays=.25') 
          estring=$
          'go_special_movie_client,p1,p2,/sswwww_client,topurl=topurl,' + $ 
          'maxframe=maxframe,minframe=minframe,outsize=outsize,reverse_ct=reverse_ct' + $
               (["",',' + extra])(extra ne '') 

box_message,estring
          estat=execute(estring)
box_message,topurl,nbox=2
          if keyword_set(topurl) then begin 
             topurl=str_replace(topurl, $
                '/net/vestige/DataCenter/www/htdocs/TRACE',$
                'http://vestige.lmsal.com/TRACE')
             qfile=ps1+url_encode(http_names(topurl))
             box_message,['TRACE MOVIE',topurl],nbox=2
          endif else qfile=ps1+ url_encode('TRACE Movie problem')
          set_logenv,'path_http',phttp
          set_logenv,'top_http',thttp
      endcase

      service(0) eq 'goes_event': begin
         found=0
         estring='ssws_gevevent,p1, p2, ot1, ot2, found=found, /biggest' $
              + (["",',' + extra])(extra ne '') + $
                (["",',' + threshold])(threshold ne '')
         box_message,['Estring',estring]
         estat=execute(estring)
         if found then begin 
            if total(abs(exp_times_out)) gt 0 then begin 
               box_message,'Expanding output times'
               help,ot1,ot2
               time_window,[ot1,ot2],ot1,ot2,minutes=exp_times_out
               help,ot1,ot2
               help,exp_times_out
            endif
            qfile=ps1+url_encode(ot1(0))+'&'+ps2+url_encode(ot2(0))
         endif
      endcase
      service(0) eq 'image_reference': begin 
          helio=strtrim(p2,2)
          if strlen(helio) eq 6 then begin  
           estring='ssw_fake_index,anytim(p1,/ccsds),helio=helio,fov=fov,index'
           estat=execute(estring)
           if estat then begin 
              ofile=concat_dir(outdir,ff+'.geny') 
              savegenx,index, file=ofile
              qfile=ps1+url_encode(http_names(ofile))

           endif else qfile=ps1+'IMAGE_REFERENCE Execution Problem' 
          
          endif else qfile=ps1+'HELIOCOORD Problem'

      endcase
      is_member(service(0),['plot_map','plot_map_gui']): begin
box_message,'PLOT_MAP OR PLOT_MAP_GUI'
          loc=http_names(p1)
          if file_exist(loc) then begin 
             box_message,'File is Local...'
             estat=execute('mreadfits,loc,index,data')
box_message,loc
help,estat
          endif else  begin 
             estat=execute('ssw_url2data,p1,index,data')
          endelse
help,index,data
          
          if data_chk(index,/struct) then begin 
             set_plot,'z'
             wdef,xx,str2number(xsize)>256,str2number(ysize)>256,/zbuffer
             loadct,0
             index2map,index,data,map
             if keyword_set(image_reference) then begin 
                if strpos(image_reference,'http') eq -1 then begin 
                   box_message,image_reference
                   ssw_fake_index,xx,gui_ref=image_reference,image_reference
                endif else begin 
                   local=http_names(image_reference)
                   if file_exist(local) then begin
                      box_message,'Image reference local, no transfer required'
                      restgenx,file=local,image_reference
                   endif
                endelse
             endif
             if required_tags(image_reference,'naxis1,naxis2,crpix1') then begin  
                refind=image_reference
                index2map,refind,bytarr(refind.naxis1,refind.naxis2),refmap
                sub_map,map,smap,ref_map=refmap
                map=temporary(smap)
             endif
             estring='plot_map,map' + (["",',' + extra])(extra ne '') 
             estat=execute(estring)
             pngfile=concat_dir(outdir,ff+'.png')
             zbuff2file2,pngfile
             if service(0) eq 'plot_map_gui' then begin 
                ofile=concat_dir(outdir,ff+'.html')
                html_doc,ofile,/header, $
                base='http://www.lmsal.com/solarsoft/ssw_client/data/'
                delvarx,jsgui
                cache_data,'jsgui',p1,jsgui,/get,status=status
                if not status then begin
                   estring='jsgui=' + $
                   'make_plot_map_select_js(map,pngfile=pngfile,' + $
                   'orig_fits=p1,ssw_service="ssw_service_plot_map_png.sh")'
                   status=execute(estring)
                   box_message,['Generating gui javascript',estring]
                endif else box_message,'Using cached GUI javascript'
                if data_chk(jsgui,/string) then begin
                   file_append,ofile,jsgui 
                   cache_data,'jsgui',p1,jsgui ; cache javascript
                endif else $
                   file_append,ofile,'PROBLEM with map->js conversion'
                html_doc,ofile,/trailer
             endif else ofile=concat_dir(outdir,pngfile)
             if strpos(ofile,'//') ne -1 then $
                ofile='/' + ssw_strsplit(ofile,'//',/tail,/last)
             qfile=ps1+url_encode(http_names(ofile))
          endif
      endcase
      service(0) eq 'get_nar': begin 
         estring='nar=get_nar(p1,p2,/uniq,count=ncount)'
         estat=execute(estring)
         if estat then estat = execute('narvo=ssw_nar2votable(nar)')
         ofile='NAR Problem'
         if estat then begin 
            ofile=concat_dir(outdir,ff+'.xml')
            file_append,ofile,narvo,/new
         endif
         qfile=ps1+url_encode(http_names(ofile))
       endcase

      service(0) eq 'get_cmelist': begin 
         count=0
         estring='cmelist=vobs_struct2votable2(ssw_getcme_list(p1,p2,/cdaw' + $
                    (["",',' + extra])(extra ne '') + '))'
         estat=execute(estring)
         ofile='No matching CMEs'
         if estat then begin 
            ofile=concat_dir(outdir,ff+'.xml')
            file_append,ofile,cmelist,/new
         endif
         qfile=ps1+url_encode(http_names(ofile))
      endcase

      service(0) eq 'coalign': begin
      endcase


      service(0) eq 'plot_rstn_srs': begin
          wdef,xx,768,512,/zbuffer
          estring="plot_rstn_srs,/nops,'"+ p1+"','" +p2+"'" + $
             tstyle + trange
          estring=estring+ (["",',' + extra])(extra ne '')
print,'>>> ' + estring
             estat=execute(estring)
             ofile=concat_dir(outdir,ff+'.png')
             zbuff2file2,ofile
             qfile=ps1+url_encode(http_names(ofile))

      endcase
      service(0) eq 'goes_xrays': begin
          wdef,xx,768,512,/zbuffer
          linecolors
          estring="plot_goes,back=11,color=7,gcolor=150,'"+ p1+"','" +p2+"'" + $
             tstyle + trange
          estring=estring+ (["",',' + extra])(extra ne '')
print,'>>> ' + estring
             estat=execute(estring)
             ofile=concat_dir(outdir,ff+'.png')
             zbuff2file2,ofile
             qfile=ps1+url_encode(http_names(ofile))

      endcase
      service(0) eq 'goes_particles': begin 
          set_plot,'z'
          estring="plot_goesp,'"+ p1+"','" +p2+"'" + $
             tstyle + trange
          estring=estring+ (["",',' + extra])(extra ne '')
             estat=execute(estring) 
             ofile=concat_dir(outdir,ff+'.png')
             zbuff2file2,ofile
             qfile=ps1+url_encode(http_names(ofile))
      endcase

      service(0) eq 'plot_ace': begin 
          set_plot,'z'
          estring="plot_ace,'"+ p1+"','" +p2+"'" + tstyle + trange + $
             (['',',/'+inst_wave])(inst_wave ne '')
          estring=estring+ (["",',' + extra])(extra ne '')
            estat=execute(estring) 
             ofile=concat_dir(outdir,ff+'.png')
             zbuff2file2,ofile
             qfile=ps1+url_encode(http_names(ofile))
      endcase
      service(0) eq 'rebin_graphic': begin 
           break_url,p1,server1,path1,files1
           break_file,files1,logical,fpath,fname,fext,fver
           sock_copy,p1,out_dir=outdir
           ofiles=concat_dir(outdir,files1(0))
           if file_exist(ofiles(0)) then begin
              estat=execute('img=read_image(ofiles(0),r,g,b)')
              factor=gt_tagval(info,/FACTOR,missing="")
              if is_number(factor) then factor= $
                   strtrim(str2number(factor)<200,2)      ; limit OUTSIZ
              nx=gt_tagval(info,/NX,missing="")
              if is_number(nx) then nx = $                ; ditto
                   strtrim(str2number(nx)<4096,2)
              estring='rebinimg=mkthumb(img,r,g,b,factor=factor,nx=nx' $
                     +(["",extra])(extra ne "") + ')'
              if data_chk(img,/nimage) ge 1 then $
                  estat=execute(estring) 
              if data_chk(rebinimg,/nimage) ge 1 then begin
                 tvlct,rr,gg,bb,/get
                 rname=concat_dir(outdir,fname+'_rebin.png') 
                 zbuff2file2,rname,r,g,b
                 tvlct,rr,gg,bb
                 qfile=ps1+url_encode(http_names(rname))  
                 
              endif else qfile='PROBLEM1' 
           endif else qfile='PROBLEM2'
       endcase


      else: begin
      endcase
   endcase
      if netdisp ne '' and file_exist(ofile) then begin 
         estring='netscape_control,"'+$
            http_names(ofile)+'",/openurl,display="'+netdisp+'"'
         estat=execute(estring)
      endif
   if post2 ne '' and cosec_mode ne '2' then begin
      box_message,'Posting result...'
      if debug then stop,'post'
      box_message,['POST2> ' + post2,'QUERY> ' + qfile]
      ssw_post_query,post2,qfile, status=status      ,/get
   endif
   case 1 of 
      cosec_mode eq '2' or cosec_mode eq '3': begin 
         estat=execute('qdecode=url_decode(qfile)')
         if data_chk(qdecode,/struct) then begin  
         outvals=strarr(n_tags(qdecode))
         params=tag_names(qdecode)
         for tt=0,n_tags(qdecode)-1 do $
               outvals(tt)=strtrim(qdecode.(tt),2)
         if data_chk(qdecode,/struct) then begin 
            xmlout=['<result>',$
                    '<param name="'+ params +'">' + $
                     outvals + '</param>',           $
                    '</result>' ]
            
            help,qdecode,/str
            print,http_names(response_file)
            if debug then stop
            if no_xml then begin 
               box_message,['NOXML',outvals(0)]
               file_append,response_file,rd_tfile(http_names(outvals(0))),/new
            endif else $
            file_append,response_file,['<html><body>',$
                 xmlout,               '</body></html>'],/new
          endif else box_message,'Problem with output...'
          endif else box_message,'Query problem or no such service'
      endcase
      else:
   endcase 
endelse

return
end
