pro ssw_service_site_registry, cgi_url_parent, cgi_path_parent, outdir=outdir, $
   www_register=www_register, debug=debug
;
;+
;   Name: ssw_service_site_registry
;
;   Purpose: make table of locally defined ssw cgi services
;
;   Input Parameters:
;      cgi_url - where the service cgis reside - default=lmsal 
;                http://www.lmsal.com/solarsoft/cgi-diapason/
;      cgi_url - local nfs path equivilent of above
;
;
;   Keyword Parameters:
;      outdir - optional output directory for service table
;               Default=$SSW/site/logs/
;
;   Calling Sequence:
;      ssw_service_site_registry,cgi_url,cgi_path [,outdir=outdir]
;
;   History:
;     7-dec-2004 - S.L.Freeland
;
;  Side Effects: local registry file is updated from current cgi contents 
;
;
;   

phttp=get_logenv('path_http') ; save current env status
thttp=get_logenv('top_http')
debug=keyword_set(debug)

case n_params() of 
   0: begin 
         cgi_path_parent='/net/diapason/www2/cgi-diapason'
         cgi_url_parent='http://www.lmsal.com/solarsoft/cgi-diapason'
         if keyword_set(www_register) then www_path=$
            '/net/diapason/www1/htdocs/solarsoft/cosec/registry'
   endcase
   1: begin 
         cgi_path_parent=http_names(cgi_url_path)
         if not file_exist(cgi_path_parent) then begin 
            box_message,['Cannot derive local cgi path from cgi url; please specify 2nd param and rerun']
            return
         endif
   endcase 
   else:  
endcase

set_logenv,'path_http',cgi_path_parent
set_logenv,'top_http',cgi_url_parent

if not file_exist(cgi_path_parent) then begin 
   box_message,'Cannot find local cgi area: ' + cgi_path_parent
   return
endif

ssw_services=file_list(cgi_path_parent,'ssw_*service*.sh')

if ssw_services(0) eq '' then begin 
   box_message,'No ssw service cgi files found....'
   return
endif

nserv=n_elements(ssw_services)
serv_urls=strarr(nserv)
server_ids=strarr(nserv)
service_hosts=strarr(nserv)

for i=0,nserv-1 do begin 
   cgi=rd_tfile(ssw_services(i))
   lcgi=strlowcase(cgi)
   ssrpc=where(strpos(lcgi,'idlrpc') ne -1,sscnt)
   if sscnt gt 0 then begin 
      cols=str2cols(lcgi(ssrpc(0)),/trim,/unali)
      server_ids(i)=cols(1)
      service_hosts(i)=cols(2)
   endif else box_message,'?? service> ' + ssw_services(i) 
endfor

ssvalid=where(service_hosts ne '' and (strpos(ssw_services,'_xxx.sh') eq -1),vss)

if vss eq 0 then begin 
   box_message,'No valid services identified; registry not updated'
   return
endif

ssw_services=http_names(ssw_services(ssvalid))
server_ids=server_ids(ssvalid)
service_hosts=service_hosts(ssvalid)

set_logenv,'top_http',thttp  ; restore status
set_logenv,'path_http',phttp 

if not keyword_set(outdir) then outdir=get_logenv('SSW_SITE_SETUP')
root_name=str_replace(str_replace(str_replace(str_replace(cgi_url_parent,'http://',''),'-','_'),'/','_'),'.','_')
serv_dbase=concat_dir(outdir,'ssw_services_'+root_name)
savegenx,file=serv_dbase,ssw_services,server_ids,service_hosts,/over
if file_exist(www_path) then begin 
   www_dbase=concat_dir(www_path,'ssw_services_'+root_name)
   savegenx,file=www_dbase,ssw_services,server_ids,service_hosts,/over
endif 
if debug then stop
return
end
