pro sxt_wwwmovie_l, qfile=qfile, gif_file=gif_file, outfil=outfil, $
		  background=background
;+
;   Name: sxt_wwwmovie
;
;   Purpose: make SXT movies from WWW requests
;
;   Keyword Parameters:
;      qfile - WWW Post Query filename  
;      background - if set, running in background (Q request)
;                   (see sxt_wwwmonitor.pro)
;  
;   History:
;      8-jul-1996 S.L.Freeland
;     10-jul-1996 S.L.Freeland - invert plot_trav and scale size for Z-buff
;     12-may-1997 S.L.Freeland - add Q/background ability
;                                optimize forground task  
;     29-may-1997 S.L.Freeland - protect against zero files 
;     15-jul-1997 S.L.Freeland - add /JAVA hook in image2mpeg call
;     22-jul-1997 S.L.Freeland - add /STILL hook
;     10-oct-1997 S.L.Freeland - generate FORM if no e-mail
;                                (allow user to add e-mail after the fact)  
;     04-dec-1997 S.L.Freeland - allow multiple mailnames
;      9-Sep-1999 S.L.Freeland - add /BASE_URL to html_doc call, increased
;                                immediate job processing number  
;     29-Sep-1999 S.L.Freeland - change email->lmsal
;     12-sep-2005(!) - S.L.Freeeland - Legacy ; use FITS directly
;                      use sxt_time2legacy_files.pro 
;                      increased nrt cutoff to 100
;-
background=keyword_set(background)
foreground=1-background

; ------- read and decode the FORM input -----------
break_file,qfile,log,paths,qfil,qver,qext
box_message,qfile
mname=qfil+qver+qext
qfile=concat_dir(concat_dir(get_logenv('path_http'),'text_client'),mname)
outfil=qfile + '.text'
qstr=url_decode(qfile=qfile)

box_message,qfile
help,qstr,/str
; ------------------------------------------------

; ---------- start an HTML document ---------------------

htmlfile=str_replace(outfil,'text','html')

htmlfile=concat_dir(concat_dir('$path_http','movies'),mname+ '.html')
box_message,htmlfile

html_doc,htmlfile,/header,template='$path_http/../YPOP/header_template.html',/base_url

; Bailout if BAD UTL decode (write problem html)
if data_chk(qstr,/string) then begin
   file_append,htmlfile, ['<h2>SORRY - HAD A PROBLEM DECODING YOUR FORM</h2>', $
                          'Is this an old browser?','<hr>']
      return				; **** EARLY EXIT ****
endif

; extract start and stop times
startt=url_query2time(qstr,/start,/nodef)
stopt =url_query2time(qstr,/stop,/nodef)

adjust_times, startt,  stopt, attext, status=atstatus

; --------------- check FORM switches ------------
tags=tag_names(qstr)
switches=strupcase(str2arr('reverse,labelframes'))
opslist=[startt, stopt]
for i=0,n_elements(switches)-1 do begin
   exe=execute(switches(i) + '=(wc_where(tags,switches(i),cnt))(0) ne -1')
   opslist=[opslist,(['off','on'])(cnt gt 0)]
endfor
; -----------------------------------------------------

ptemp=!d.name				; save some status
ctemp=!p.charsize
!p.charsize=.7

!p.charsize=ctemp					
set_plot,ptemp

sxtres=gt_tagval(qstr,/sxtres,missing="ALL")
sxtwave=gt_tagval(qstr,/sxtwave,missing="ALL")

prefix=imgtype2prefix(gt_tagval(qstr,/imgtype),topdir=topdir)
prstr,/nomore,strjustify(["Prefix: " + prefix],/box)
cadence=(strlowcase(qstr.cadence))(0)
cadnum=str2number(cadence)

case 1 of 
   strpos(cadence,'hour') ne -1: hour=cadnum
   strpos(cadence,'hour') ne -1: day=cadnum
   else: day=cadnum
endcase

prstr,/nomore, strjustify(["Cadence: " + cadence + strtrim(cadnum,2) ],/box)
help,cadence,day,hour
files=sxt_time2legacy_files(startt, stopt,day=day, hour=hour, $
   filterb=sxtwave, resolution=sxtres, ftimes=ftimes)
nfiles=n_elements(files)
 
if files(0) eq '' then begin                ; ZERO files found
   mess=['Sorry, no SXT data available between: ' + startt  + ' and ' + stopt]
   file_append,htmlfile,str2html(["",mess])
endif else begin
   times=anytim(ftimes,/int)
   labels='' 

   if labelframes then labels =anytim(times,out_style='ECS',/truncate)

   prstr,labels,/nomore
   help,qstr,/str
   help,qstr.cadence
   outsize=fix((str2arr(qstr.outsize,'_'))(0))

;  determine if background job is required
   fcut=50
   scut=512

   mailname=qstr.email
   if mailname ne '' then print,mailname

   if (nfiles gt fcut or outsize gt scut) and foreground then begin
      mess=["Your movie request is too large for immediate processing...", $
	"we will queue this request and run asap","",$, $
        "Future URL will be: ","",http_names(htmlfile)]
       file_append,htmlfile,str2html(mess)
       qname=concat_dir('$SSW_SITE_SETUP','wwwmovieq.dat')
       file_append,qname,qfile
;      build a FORM to allow post-movie request e-mail submission
       if strtrim(mailname,2) eq '' then begin
           mess= $
	   ["If you supply email address and submit, we can remind you ", $
	   " of the URL when the movie has completed", $
           '<FORM METHOD="POST" ACTION="../../cgi-diapason/sxtmovie_addname.sh">',  $
	   '<em>Notify >></em>:<input type="text" name="email" size=34></textarea>', $
	   ' <em>when >></em>: ', $
	   '<select name="qfile"><option selected> ' + qfil+qver+ '</select> is ready' , $
           '<em><INPUT TYPE="submit" VALUE="Submit e-mail address"></em><p>'] 
       endif else  mess=["We will notify: <em><b>" + mailname + " </b></em> when movie has completed"]	   
       file_append,htmlfile,str2html(["",mess])
       mail, [qfile,mess], user='freeland', /no_def, subj="SXT WWW JOB QUEUED"
   endif else begin
      if background  then print,"running as background job..."
      bigcut=250
      toobig=nfiles gt bigcut
      tbmess=''
      if toobig then begin
         tbmess=["More than " + strtrim(bigcut,2) + " images were requested", $
	     "truncating to that number "]     
         prstr,strjustify(tbmess,/box),/nomore
         files=files(0:bigcut-1)
         times=times(0:bigcut-1)
         if labelframes then labels=labels(0:bigcut-1)
      endif 
      mreadfits,files,index,data
      print,'------- calling image2movie -------'
      
      image2movie, data, uttimes=index, outsize=outsize, $
          gamma=qstr.gamma, reverse=reverse, table=qstr.colortable, $
          fmt_tim='ECS',html=html, /inctime, thumbsize=100, $
          mpeg=(qstr.type eq 'mpeg'), movie_name=mname+'X', $
          movie_dir=concat_dir('path_http','movies'), $
          java=(qstr.type eq 'JavaScript'), $
          still=(qstr.type eq 'Still Sequence (GIF)'), $
	  loop=(strpos(qstr.type,'loop') ne -1),label=labels, $
	     server=foreground
      print,'------- done image2movie -------'
      prstr,html,/nomore
      file_append,htmlfile, ['<p>',html]
   endelse
endelse
mess=systime() + " all done, returning..."

html_doc,htmlfile,/trailer,template='$path_http/../YPOP/trailer_template.html'

if background then begin
   maillist=['freeland@sxt1.lmsal.com']
   mess=['Your SXT WWW request has completed', $
	 'The URL is:', $
	 '   ' + http_names(htmlfile), '',tbmess,'']
   if mailname eq '' then begin
      addname=concat_dir('$SSW_SITE_SETUP','wwwmovie_addname.dat')
      newnames=rd_tfile(addname,nocomment=';')
      ssadd=where(strpos(newnames,qfil+qver) ne -1,fcnt)

      if fcnt gt 0 then begin
          mailname=strtrim(str_replace(newnames(ssadd),qfil+qver,''),2)
          mess=[mess,"E-mail address from: wwwmovie_addname"]
      endif else mess=[mess,'*** NO e-mail ADDRESS SUPPLIED ***']
   endif
   mnok=where(mailname ne '',mncnt)
   if mncnt gt 0 then begin
      mailname=mailname(mnok)
      maillist=[maillist,mailname]
   endif
   mail, mess, user=maillist, /no_def, subj="SXT WWW request completed"
endif  

return
end

